@@ -2,7 +2,7 @@
 
 # ############################################################################ #
 #                                                                              #
-#   Copyleft (C) 2020-2024, Marek Gagolewski <https://www.gagolewski.com>      #
+#   Copyleft (C) 2020-2025, Marek Gagolewski <https://www.gagolewski.com>      #
 #                                                                              #
 #                                                                              #
 #   This program is free software: you can redistribute it and/or modify       #
@@ -23,7 +23,7 @@
 .correct_height <- function(height)
 {
     # correction for the departure from ultrametricity
-    # TODO: allow one choose?
+    # TODO: allow one to choose?
     # cumsum(height)
     rev(cummin(rev(height)))
 }
@@ -36,50 +36,51 @@
 #'
 #' @description
 #' A reimplementation of \emph{Genie} - a robust and outlier resistant
-#' clustering algorithm (see Gagolewski, Bartoszuk, Cena, 2016).
-#' The Genie algorithm is based on a minimum spanning tree (MST) of the
+#' clustering algorithm by Gagolewski, Bartoszuk, and Cena (2016).
+#' The Genie algorithm is based on the minimum spanning tree (MST) of the
 #' pairwise distance graph of a given point set.
 #' Just like the single linkage, it consumes the edges
 #' of the MST in an increasing order of weights. However, it prevents
 #' the formation of clusters of highly imbalanced sizes; once the Gini index
 #' (see \code{\link{gini_index}()}) of the cluster size distribution
-#' raises above \code{gini_threshold}, a forced merge of a point group
-#' of the smallest size is performed. Its appealing simplicity goes hand
-#' in hand with its usability; Genie often outperforms
-#' other clustering approaches on benchmark data,
-#' such as \url{https://github.com/gagolews/clustering-benchmarks}.
+#' raises above \code{gini_threshold}, the merging of a point group
+#' of the smallest size is enforced.
 #'
-#' The clustering can now also be computed with respect to the
-#' mutual reachability distance (based, e.g., on the Euclidean metric),
+#' The clustering can also be computed with respect to the
+#' mutual reachability distances (based, e.g., on the Euclidean metric),
 #' which is used in the definition of the HDBSCAN* algorithm
-#' (see Campello et al., 2013). If \code{M} > 1, then the mutual reachability
-#' distance \eqn{m(i,j)} with smoothing factor \code{M} is used instead of the
-#' chosen "raw" distance \eqn{d(i,j)}. It holds \eqn{m(i,j)=\max(d(i,j), c(i), c(j))},
-#' where \eqn{c(i)} is \eqn{d(i,k)} with \eqn{k} being the
-#' (\code{M}-1)-th nearest neighbour of \eqn{i}.
-#' This makes "noise" and "boundary" points being "pulled away" from each other.
+#' (see Campello et al., 2013). If \eqn{M>1}, then the mutual reachability
+#' distance \eqn{m(i,j)} with a smoothing factor \eqn{M} is used instead of the
+#' chosen "raw" distance \eqn{d(i,j)}.  It holds
+#' \eqn{m(i,j)=\max(d(i,j), c(i), c(j))}, where the core distance \eqn{c(i)} is
+#' the distance to the \eqn{i}-th point's (\eqn{M-1})-th
+#' nearest neighbour.  This makes "noise" and "boundary" points being
+#' more "pulled away" from each other.
 #'
-#' The Genie correction together with the smoothing factor \code{M} > 1 (note that
-#' \code{M} = 2 corresponds to the original distance) gives a robustified version of
-#' the HDBSCAN* algorithm that is able to detect a predefined number of
-#' clusters. Hence it does not dependent on the DBSCAN's somewhat magical
+#' The Genie correction together with the smoothing factor \eqn{M>1}
+#' (note that \eqn{M=2} corresponds to the original distance) gives
+#' a version of the HDBSCAN* algorithm that is able to detect
+#' a predefined number of clusters. Hence it does not dependent on the DBSCAN's
 #' \code{eps} parameter or the HDBSCAN's \code{min_cluster_size} one.
 #'
 #'
 #' @details
-#' Note that, as in the case of all the distance-based methods,
+#' As in the case of all the distance-based methods,
 #' the standardisation of the input features is definitely worth giving a try.
+#' Oftentimes, more sophisticated feature engineering (e.g., dimensionality
+#' reduction) will lead to more meaningful results.
 #'
 #' If \code{d} is a numeric matrix or an object of class \code{dist},
 #' \code{\link{mst}()} will be called to compute an MST, which generally
-#' takes at most \eqn{O(n^2)} time (the algorithm we provide is parallelised,
-#' environment variable \code{OMP_NUM_THREADS} controls the number of threads
-#' in use). However, see \code{\link{emst_mlpack}()} for a very fast alternative
-#' in the case of Euclidean spaces of (very) low dimensionality and \code{M} = 1.
+#' takes at most \eqn{O(n^2)} time. However, by default, a faster algorithm
+#' based on K-d trees is selected automatically for low-dimensional Euclidean
+#' spaces; see \code{\link[quitefastmst]{mst_euclid}} from
+#' the \pkg{quitefastmst} package.
 #'
-#' Given an minimum spanning tree, the algorithm runs in \eqn{O(n \sqrt{n})} time.
-#' Therefore, if you want to test different \code{gini_threshold}s,
-#' (or \code{k}s), it is best to explicitly compute the MST first.
+#' Once a minimum spanning tree is determined, the Genie algorithm runs in
+#' \eqn{O(n \sqrt{n})} time.  If you want to test different
+#' \code{gini_threshold}s or \code{k}s,  it is best to compute
+#' the MST first explicitly.
 #'
 #' According to the algorithm's original definition,
 #' the resulting partition tree (dendrogram) might violate
@@ -91,87 +92,87 @@
 #'
 #' @param d a numeric matrix (or an object coercible to one,
 #'     e.g., a data frame with numeric-like columns) or an
-#'     object of class \code{dist}, see \code{\link[stats]{dist}}
-#'     or an object of class \code{mst}, see \code{\link{mst}()}.
+#'     object of class \code{dist} (see \code{\link[stats]{dist}}),
+#'     or an object of class \code{mst} (\code{\link{mst}})
 #' @param gini_threshold threshold for the Genie correction, i.e.,
 #'     the Gini index of the cluster size distribution;
-#'     Threshold of 1.0 disables the correction.
-#'     Low thresholds highly penalise the formation of small clusters.
+#'     threshold of 1.0 leads to the single linkage algorithm;
+#'     low thresholds highly penalise the formation of small clusters
 #' @param distance metric used to compute the linkage, one of:
 #'     \code{"euclidean"} (synonym: \code{"l2"}),
 #'     \code{"manhattan"} (a.k.a. \code{"l1"} and \code{"cityblock"}),
-#'     \code{"cosine"}.
+#'     \code{"cosine"}
 #' @param verbose logical; whether to print diagnostic messages
-#'     and progress information.
-#' @param cast_float32 logical; whether to compute the distances using 32-bit
-#'     instead of 64-bit precision floating-point arithmetic (up to 2x faster).
-#' @param ... further arguments passed to other methods.
-#' @param k the desired number of clusters to detect, \code{k} = 1 with \code{M} > 1
-#'     acts as a noise point detector.
+#'     and progress information
+#' @param ... further arguments passed to \code{\link{mst}()}
+#' @param k the desired number of clusters to detect, \eqn{k=1} with
+#'      \eqn{M>1} acts as a noise point detector
 #' @param detect_noise whether the minimum spanning tree's leaves
-#'     should be marked as noise points, defaults to \code{TRUE} if \code{M} > 1
-#'     for compatibility with HDBSCAN*.
-#' @param M smoothing factor; \code{M} <= 2 gives the selected \code{distance};
-#'     otherwise, the mutual reachability distance is used.
+#'     should be marked as noise points, defaults to \code{TRUE} if \eqn{M>1}
+#'     for compatibility with HDBSCAN*
+#' @param M smoothing factor; \eqn{M \leq 2} gives the selected \code{distance};
+#'     otherwise, the mutual reachability distance is used
 #' @param postprocess one of \code{"boundary"} (default), \code{"none"}
-#'     or \code{"all"};  in effect only if \code{M} > 1.
+#'     or \code{"all"};  in effect only if \eqn{M > 1}.
 #'     By default, only "boundary" points are merged
 #'     with their nearest "core" points (A point is a boundary point if it is
-#'     a noise point and it's amongst its adjacent vertex's
-#'     \code{M}-1 nearest neighbours). To force a classical
+#'     a noise point and it is amongst its adjacent vertex's
+#'     (\eqn{M-1})-th nearest neighbours). To force a classical
 #'     k-partition of a data set (with no notion of noise),
-#'     choose "all".
+#'     choose \code{"all"}
 #'
 #'
 #' @return
-#' \code{gclust()} computes the whole clustering hierarchy; it
-#' returns a list of class \code{hclust},
-#' see \code{\link[stats]{hclust}}. Use \code{\link[stats]{cutree}} to obtain
-#' an arbitrary k-partition.
+#' \code{gclust()} computes the entire clustering hierarchy; it
+#' returns a list of class \code{hclust}; see \code{\link[stats]{hclust}}.
+#' Use \code{\link[stats]{cutree}} to obtain an arbitrary \code{k}-partition.
+#'
+#' \code{genie()} returns a \code{k}-partition - a vector whose i-th element
+#' denotes the i-th input point's cluster label between 1 and \code{k}
+#' If \code{detect_noise} is \code{TRUE}, missing values (\code{NA}) denote
+#' noise points.
 #'
-#' \code{genie()} returns a \code{k}-partition - a vector with elements in 1,...,k,
-#' whose i-th element denotes the i-th input point's cluster identifier.
-#' Missing values (\code{NA}) denote noise points (if \code{detect_noise}
-#' is \code{TRUE}).
 #'
 #' @seealso
-#' \code{\link{mst}()} for the minimum spanning tree routines.
+#' \code{\link{mst}()} for the minimum spanning tree routines
 #'
-#' \code{\link{adjusted_rand_score}()} (amongst others) for external
-#' cluster validity measures (partition similarity scores).
+#' \code{\link{normalized_clustering_accuracy}()} (amongst others) for external
+#' cluster validity measures
 #'
 #'
 #' @references
-#' Gagolewski M., Bartoszuk M., Cena A.,
+#' Gagolewski, M., Bartoszuk, M., Cena, A.,
 #' Genie: A new, fast, and outlier-resistant hierarchical clustering algorithm,
 #' \emph{Information Sciences} 363, 2016, 8-23,
-#' \doi{10.1016/j.ins.2016.05.003}.
+#' \doi{10.1016/j.ins.2016.05.003}
 #'
-#' Campello R.J.G.B., Moulavi D., Sander J.,
+#' Campello, R.J.G.B., Moulavi, D., Sander, J.,
 #' Density-based clustering based on hierarchical density estimates,
 #' \emph{Lecture Notes in Computer Science} 7819, 2013, 160-172,
-#' \doi{10.1007/978-3-642-37456-2_14}.
+#' \doi{10.1007/978-3-642-37456-2_14}
 #'
-#' Gagolewski M., Cena A., Bartoszuk M., Brzozowski L.,
+#' Gagolewski, M., Cena, A., Bartoszuk, M., Brzozowski, L.,
 #' Clustering with minimum spanning trees: How good can it be?,
-#' \emph{Journal of Classification}, 2024, in press,
-#' \doi{10.1007/s00357-024-09483-1}.
+#' \emph{Journal of Classification} 42, 2025, 90-112,
+#' \doi{10.1007/s00357-024-09483-1}
 #'
 #'
 #' @examples
 #' library("datasets")
 #' data("iris")
-#' X <- iris[1:4]
+#' X <- jitter(as.matrix(iris[2:3]))
 #' h <- gclust(X)
 #' y_pred <- cutree(h, 3)
-#' y_test <- iris[,5]
-#' plot(iris[,2], iris[,3], col=y_pred,
-#'    pch=as.integer(iris[,5]), asp=1, las=1)
+#' y_test <- as.integer(iris[,5])
+#' plot(X, col=y_pred, pch=y_test, asp=1, las=1)
 #' adjusted_rand_score(y_test, y_pred)
-#' pair_sets_index(y_test, y_pred)
+#' normalized_clustering_accuracy(y_test, y_pred)
 #'
-#' # Fast for low-dimensional Euclidean spaces:
-#' # h <- gclust(emst_mlpack(X))
+#' y_pred2 <- genie(X, 3, M=5)  # clustering wrt 5-mutual reachability distance
+#' plot(X[,1], X[,2], col=y_pred2, pch=y_test, asp=1, las=1)
+#' noise <- is.na(y_pred2)  # noise/boundary points
+#' points(X[noise, ], col="gray", pch=10)
+#' normalized_clustering_accuracy(y_test[!noise], y_pred2[!noise])
 #'
 #' @rdname gclust
 #' @export
@@ -188,16 +189,13 @@ gclust <- function(d, ...)
 gclust.default <- function(d,
     gini_threshold=0.3,
     distance=c("euclidean", "l2", "manhattan", "cityblock", "l1", "cosine"),
-    cast_float32=TRUE,
     verbose=FALSE, ...)
 {
-    stopifnot(gini_threshold >= 0.0, gini_threshold <= 1.0)
     distance <- match.arg(distance)
-    d <- as.matrix(d)
-
-    gclust.mst(mst.default(d, M=1L, distance=distance,
-                verbose=verbose, cast_float32=cast_float32),
-        gini_threshold=gini_threshold, verbose=verbose)
+    gclust.mst(
+        mst.default(d, distance=distance, verbose=verbose, ...),
+        gini_threshold=gini_threshold, verbose=verbose
+    )
 }
 
 
@@ -208,20 +206,25 @@ gclust.dist <- function(d,
     gini_threshold=0.3,
     verbose=FALSE, ...)
 {
-    stopifnot(gini_threshold >= 0.0, gini_threshold <= 1.0)
-    gclust.mst(mst.dist(d, M=1L, verbose=verbose),
-        gini_threshold=gini_threshold, verbose=verbose)
+    gclust.mst(
+        mst.dist(d, verbose=verbose, ...),
+        gini_threshold=gini_threshold, verbose=verbose
+    )
 }
 
 
 #' @export
 #' @rdname gclust
 #' @method gclust mst
-gclust.mst <- function(d,
+gclust.mst <- function(
+    d,
     gini_threshold=0.3,
     verbose=FALSE, ...)
 {
+    gini_threshold <- as.double(gini_threshold)[1]
     stopifnot(gini_threshold >= 0.0, gini_threshold <= 1.0)
+    verbose <- !identical(verbose, FALSE)
+
     result <- .gclust(d, gini_threshold, verbose)
 
     result[["height"]] <- .correct_height(result[["height"]])
@@ -235,8 +238,6 @@ gclust.mst <- function(d,
 }
 
 
-
-
 #' @rdname gclust
 #' @export
 genie <- function(d, ...)
@@ -248,52 +249,52 @@ genie <- function(d, ...)
 #' @export
 #' @rdname gclust
 #' @method genie default
-genie.default <- function(d,
+genie.default <- function(
+    d,
     k,
     gini_threshold=0.3,
     distance=c("euclidean", "l2", "manhattan", "cityblock", "l1", "cosine"),
     M=1L,
     postprocess=c("boundary", "none", "all"),
     detect_noise=M>1L,
-    cast_float32=TRUE,
-    verbose=FALSE, ...)
+    verbose=FALSE,
+    ...)
 {
-    stopifnot(gini_threshold >= 0.0, gini_threshold <= 1.0)
-    postprocess <- match.arg(postprocess)
     distance <- match.arg(distance)
-
-    d <- as.matrix(d)
-
-    genie.mst(mst.default(d, M=M, distance=distance,
-                verbose=verbose, cast_float32=cast_float32),
+    postprocess <- match.arg(postprocess)
+    genie.mst(
+        mst.default(d, M=M, distance=distance, verbose=verbose, ...),
         k=k,
         gini_threshold=gini_threshold,
         postprocess=postprocess,
         detect_noise=detect_noise,
-        verbose=verbose)
+        verbose=verbose
+    )
 }
 
 
 #' @export
 #' @rdname gclust
 #' @method genie dist
-genie.dist <- function(d,
+genie.dist <- function(
+    d,
     k,
     gini_threshold=0.3,
     M=1L,
     postprocess=c("boundary", "none", "all"),
     detect_noise=M>1L,
-    verbose=FALSE, ...)
+    verbose=FALSE,
+    ...)
 {
-    stopifnot(gini_threshold >= 0.0, gini_threshold <= 1.0)
     postprocess <- match.arg(postprocess)
-
-    genie.mst(mst.dist(d, M=M, verbose=verbose),
+    genie.mst(
+        mst.dist(d, M=M, verbose=verbose, ...),
         k=k,
         gini_threshold=gini_threshold,
         postprocess=postprocess,
         detect_noise=detect_noise,
-        verbose=verbose)
+        verbose=verbose
+    )
 }
 
 
@@ -305,10 +306,14 @@ genie.mst <- function(d,
     gini_threshold=0.3,
     postprocess=c("boundary", "none", "all"),
     detect_noise=FALSE,
-    verbose=FALSE, ...)
+    verbose=FALSE,
+    ...)
 {
+    gini_threshold <- as.double(gini_threshold)[1]
     stopifnot(gini_threshold >= 0.0, gini_threshold <= 1.0)
     postprocess <- match.arg(postprocess)
+    verbose <- !identical(verbose, FALSE)
+    detect_noise <- !identical(detect_noise, FALSE)
 
     structure(
         .genie(d, k, gini_threshold, postprocess, detect_noise, verbose),
