@@ -2,7 +2,7 @@
 
 # ############################################################################ #
 #                                                                              #
-#   Copyleft (C) 2020-2023, Marek Gagolewski <https://www.gagolewski.com>      #
+#   Copyleft (C) 2020-2025, Marek Gagolewski <https://www.gagolewski.com>      #
 #                                                                              #
 #                                                                              #
 #   This program is free software: you can redistribute it and/or modify       #
@@ -24,16 +24,13 @@
 #' @description
 #' See \code{\link{genie}()} for more details.
 #'
-#'
-#' @name genieclust-package
-#' @rdname genieclust-package
-#' @aliases genieclust
-#' @docType package
-#'
 #' @useDynLib genieclust, .registration=TRUE
 #' @importFrom Rcpp evalCpp
 #' @importFrom stats hclust
 #' @importFrom stats cutree
+#' @importFrom quitefastmst knn_euclid
+#' @importFrom quitefastmst mst_euclid
 #' @importFrom stats dist
 #' @importFrom utils capture.output
-invisible(NULL)
+#' @keywords internal
+"_PACKAGE"
