@@ -2,10 +2,10 @@
  *
  *  Code originally contributed in <https://github.com/gagolews/optim_cvi>,
  *  see https://doi.org/10.1016/j.ins.2021.10.004.
- *  Copyleft (C) 2020, Maciej Bartoszuk <http://bartoszuk.rexamine.com>
+ *  Copyleft (C) 2020, Maciej Bartoszuk
  *
  *  For the 'genieclust' version:
- *  Copyleft (C) 2020-2024, Marek Gagolewski <https://www.gagolewski.com>
+ *  Copyleft (C) 2020-2025, Marek Gagolewski <https://www.gagolewski.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Affero General Public License
@@ -100,7 +100,7 @@ public:
     virtual FLOAT_T compute()
     {
         FLOAT_T max_denominator = 0.0;
-        FLOAT_T min_numerator = INFTY;
+        FLOAT_T min_numerator = INFINITY;
         for (size_t i=0; i<K; ++i) {
             FLOAT_T denom_i = denominatorDelta->compute(i);
             if (denom_i > max_denominator)
@@ -179,7 +179,7 @@ public:
     virtual FLOAT_T compute()
     {
         FLOAT_T max_denominator = 0.0;
-        FLOAT_T min_numerator = INFTY;
+        FLOAT_T min_numerator = INFINITY;
         for (size_t i=0; i<K; ++i) {
             FLOAT_T denom_i = denominatorDelta->compute(i);
             if (denom_i > max_denominator)
