@@ -2,10 +2,10 @@
  *
  *  Code originally contributed in <https://github.com/gagolews/optim_cvi>,
  *  see https://doi.org/10.1016/j.ins.2021.10.004.
- *  Copyleft (C) 2020, Maciej Bartoszuk <http://bartoszuk.rexamine.com>
+ *  Copyleft (C) 2020, Maciej Bartoszuk
  *
  *  For the 'genieclust' version:
- *  Copyleft (C) 2020-2022, Marek Gagolewski <https://www.gagolewski.com>
+ *  Copyleft (C) 2020-2025, Marek Gagolewski <https://www.gagolewski.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Affero General Public License
@@ -57,8 +57,8 @@ public:
           centroids(centroids)
     { }
 
-    virtual void before_modify(size_t i, Py_ssize_t j) = 0;
-    virtual void after_modify(size_t i, Py_ssize_t j) = 0;
+    virtual void before_modify(size_t i, Py_ssize_t /*j*/) = 0;
+    virtual void after_modify(size_t i, Py_ssize_t /*j*/) = 0;
     virtual void undo() = 0;
     virtual void recompute_all() = 0;
 
